class Graph(object):
    '''
        graph in the form
        {
            node = [[nodeA, weigh1], [nodeB, weigh2]],
            nodeA = [[nodeC, weigh3]]
        }
    '''
    def __init__(self):
        self.graph = {}

    def __init__(self, graph):
        self.graph = graph

    def add_node(self, node):
        if node not in self.graph:
            self.graph[node] = []

    def add_edge(self, node_start, node_end, weight):
        edges = self.graph[node_start]
        for item in edges:
            if item[0] == node_end:
                raise ValueError("Edge already exiists")

        self.graph[node_start].append([weight, node_end])

    def delete_node(self, node):
        self.graph.pop(node, None)

    def delete_edge(self, node_start, node_end):
        edges = self.graph[node_start]
        for item in edges:
            if item[0] == node_end:
                self.graph[node_start].remove(item)

    def get_node_details(self, node):
        if node in self.graph:
            return self.graph[node]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `greedybfs` package."""


import unittest

from greedybfs.greedybfs import gbfs


class TestGreedybfs(unittest.TestCase):
    """Tests for `greedybfs` package."""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_000_something(self):
        """Test something."""
        test_graph = {
            "a": [['s',253], ['t', 329], ['z',374]],
            "s": [['f', 176], ['o', 380], ['r', 193]],
            "f": [['b', 0]]
        }
        assert gbfs('a','b',test_graph) == ['s', 'f', 'b']



# -*- coding: utf-8 -*-

from .priorityQueue import PriorityQueue
from .graph import Graph



def gbfs(start_node, end_node, graph_input={}):
    path = []
    myQueue = PriorityQueue()
    graph = Graph(graph_input)

    start_node = start_node
    end_node = end_node

    # print(start_node, end='')
    # start = graph.get_node_details(start_node)
    current_node = [start_node]

    while current_node[0] != end_node:
        current_node = graph.get_node_details(current_node[0])
        myQueue.makeEmpty()

        for item in current_node:
            myQueue.insert(item)

        current_node = myQueue.delete()
        path.append(current_node[0])
        # print("-> " + str(current_node[0]) , end = '')
    return path



class PriorityQueue(object):
    def __init__(self):
        self.queue = []

    def __str__(self):
        return ' '.join([str(i) for i in self.queue])

    # for checking if the queue is empty
    def isEmpty(self):
        return len(self.queue) == []

    # for inserting an element in the queue
    def insert(self, data):
        self.queue.append(data)

    # for popping an element based on Priority
    def delete(self):
        try:
            min = 0
            for i in range(len(self.queue)):
                if self.queue[i][1] < self.queue[min][1]:
                    min = i

            item = self.queue[min]
            del self.queue[min]
            return item

        except IndexError:
            print()
            return

    def makeEmpty(self):
        del self.queue[:]
